package com.chelvc.framework.base.apidoc;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.time.Duration;
import java.util.Map;

import com.arsframework.apidoc.core.Parameter;
import com.arsframework.apidoc.core.ParameterAnalyser;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.serializer.JacksonEnumerateSerializer;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * 自定义接口文档参数解析实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class CustomizeParameterAnalyser extends ParameterAnalyser {
    public CustomizeParameterAnalyser(Method method) {
        super(method);
    }

    /**
     * 判断参数是否需要枚举包装
     *
     * @param parameter 参数对象
     * @return true/false
     */
    private boolean isEnumerateWrapping(Parameter parameter) {
        Field field = parameter.getField();
        if (field == null) {
            JsonSerialize annotation = parameter.getOriginal().getAnnotation(JsonSerialize.class);
            return annotation != null && JacksonEnumerateSerializer.class.isAssignableFrom(annotation.using());
        }
        JsonSerialize annotation = field.getAnnotation(JsonSerialize.class);
        return annotation != null && (JacksonEnumerateSerializer.class.isAssignableFrom(annotation.using())
                || JacksonEnumerateSerializer.class.isAssignableFrom(annotation.contentUsing()));
    }

    @Override
    protected void afterInitializeParameter(Parameter parameter) {
        if (parameter.getOriginal() == Paging.class) {
            parameter.setType(Paging.class);
            parameter.setFields(null);
            parameter.setFormat("页码,分页大小");
            parameter.setExample("1,10");
        } else if (parameter.getOriginal() == Period.class) {
            parameter.setType(Period.class);
            parameter.setFields(null);
            parameter.setFormat("开始时间,结束时间");
            parameter.setExample(Period.today().toString());
        } else if (parameter.getOriginal() == Region.class) {
            parameter.setType(Integer.class);
            parameter.setFields(null);
            parameter.setExample("110000");
        } else if (parameter.getOriginal() == Duration.class) {
            parameter.setType(String.class);
            parameter.setFields(null);
            parameter.setExample("PT1S");
        } else if (!parameter.isInput() && this.isEnumerateWrapping(parameter)) {
            parameter.setType(Map.class);
            parameter.setFields(null);
            String description = StringUtils.ifEmpty(parameter.getDescription(), StringUtils.EMPTY);
            parameter.setDescription(description + "【编码/名称映射表】");
        }
    }
}
