package com.chelvc.framework.base.cache;

import java.util.List;
import java.util.Objects;

import lombok.NonNull;

/**
 * 缓存命名适配接口
 *
 * @param <Q> 查询对象类型
 * @param <V> 缓存对象类型
 * @author Woody
 * @date 2024/1/30
 */
public interface NamingAdapter<Q, V> {
    /**
     * 获取主体缓存命名
     *
     * @return 缓存命名对象实例
     */
    Naming primary();

    /**
     * 序列化查询对象
     *
     * @param object 查询对象实例
     * @return 缓存名称列表
     */
    List<String> query2string(Q object);

    /**
     * 序列化更新对象
     *
     * @param object 更新对象实例
     * @return 缓存名称列表
     */
    List<String> modify2string(V object);

    /**
     * 比较缓存对象命名是否相同
     *
     * @param object 缓存对象
     * @param other  缓存对象
     * @return true/false
     */
    default boolean equals(V object, V other) {
        return Objects.equals(object, other);
    }

    /**
     * 查询对象缓存命名适配
     *
     * @param object 对象实例
     * @return 缓存命名对象实例
     */
    default Naming querying(@NonNull Q object) {
        return this.querying(object, this.primary().getDuration());
    }

    /**
     * 查询对象缓存命名适配
     *
     * @param object   对象实例
     * @param duration 缓存有效时长（秒）
     * @return 缓存命名对象实例
     */
    default Naming querying(@NonNull Q object, long duration) {
        Naming primary = this.primary();
        return Naming.of(primary.getPrimary(), this.query2string(object), duration, primary.isReversed());
    }

    /**
     * 更新对象缓存命名适配
     *
     * @param object 对象实例
     * @return 缓存命名对象实例
     */
    default Naming modifying(@NonNull V object) {
        return this.modifying(object, this.primary().getDuration());
    }

    /**
     * 更新对象缓存命名适配
     *
     * @param object   对象实例
     * @param duration 缓存有效时长（秒）
     * @return 缓存命名对象实例
     */
    default Naming modifying(@NonNull V object, long duration) {
        Naming primary = this.primary();
        return Naming.of(primary.getPrimary(), this.modify2string(object), duration, primary.isReversed());
    }
}
