package com.chelvc.framework.base.config;

import java.util.Collections;
import java.util.List;
import javax.annotation.PostConstruct;

import com.chelvc.framework.base.interceptor.DefaultResponseWrapper;
import com.chelvc.framework.base.interceptor.ResponseWrapper;
import com.chelvc.framework.base.interceptor.ResponseWrappingInterceptor;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 请求结果统一包装配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ResponseWrappingConfigurer {
    private final RequestMappingHandlerAdapter handlerAdapter;

    @Bean
    @ConditionalOnMissingBean(ResponseWrapper.class)
    public ResponseWrapper responseWrapper() {
        return new DefaultResponseWrapper();
    }

    @Bean
    public ResponseWrappingInterceptor responseWrappingInterceptor() {
        List<HandlerMethodReturnValueHandler> handlers = handlerAdapter.getReturnValueHandlers();
        RequestResponseBodyMethodProcessor processor = CollectionUtils.isEmpty(handlers) ? null :
                handlers.stream().filter(handler -> handler instanceof RequestResponseBodyMethodProcessor)
                        .map(handler -> (RequestResponseBodyMethodProcessor) handler).findAny().orElse(null);
        return new ResponseWrappingInterceptor(this.responseWrapper(), processor);
    }

    @PostConstruct
    public void initialize() {
        ResponseWrappingInterceptor interceptor = this.responseWrappingInterceptor();
        List<HandlerMethodReturnValueHandler> handlers = this.handlerAdapter.getReturnValueHandlers();
        if (CollectionUtils.isEmpty(handlers)) {
            this.handlerAdapter.setReturnValueHandlers(Collections.singletonList(interceptor));
            return;
        }
        List<HandlerMethodReturnValueHandler> replaces = Lists.newArrayListWithCapacity(handlers.size() + 1);
        replaces.add(interceptor);
        replaces.addAll(handlers);
        this.handlerAdapter.setReturnValueHandlers(replaces);
    }
}
