package com.chelvc.framework.base.context;

import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.model.Version;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;

/**
 * 版本号上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class VersionContextHolder {
    private VersionContextHolder() {
    }

    /**
     * 判断当前版本是否是指定版本
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isVersion(@NonNull Terminal terminal, @NonNull String version) {
        Session session = SessionContextHolder.getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && version.equals(ObjectUtils.ifNull(session, Session::getVersion));
    }

    /**
     * 判断当前版本是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isBeforeVersion(@NonNull Terminal terminal, @NonNull String version) {
        return isBeforeVersion(terminal, version, false);
    }

    /**
     * 判断当前版本是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isBeforeVersion(@NonNull Terminal terminal, @NonNull String version, boolean contains) {
        Session session = SessionContextHolder.getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isBefore(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }

    /**
     * 判断当前版本是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isAfterVersion(@NonNull Terminal terminal, @NonNull String version) {
        return isAfterVersion(terminal, version, false);
    }

    /**
     * 判断当前版本是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isAfterVersion(@NonNull Terminal terminal, @NonNull String version, boolean contains) {
        Session session = SessionContextHolder.getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isAfter(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }
}
