package com.chelvc.framework.base.interceptor;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.filter.Filter;
import ch.qos.logback.core.spi.FilterReply;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 动态日志处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class DynamicLoggingInterceptor extends Filter<ILoggingEvent> {
    private Level level;

    @Override
    public FilterReply decide(ILoggingEvent event) {
        if (!this.isStarted()) {
            return FilterReply.NEUTRAL;
        }

        Level level = this.level != null ? this.level :
                ApplicationContextHolder.getSafeProperty("logging.level." + event.getLoggerName(), Level.class);
        if (level == null) {
            return FilterReply.NEUTRAL;
        } else if (event.getLevel().isGreaterOrEqual(level)) {
            return FilterReply.ACCEPT;
        }
        return FilterReply.DENY;
    }
}
