package com.chelvc.framework.common.exception;

import java.util.Map;

import com.chelvc.framework.common.model.Result;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;

/**
 * 参数无效异常
 *
 * @author Woody
 * @date 2024/1/30
 */
public class ParameterInvalidException extends FrameworkException {
    /**
     * 异常编码
     */
    public static final String CODE = "PARAMETER_INVALID";

    public ParameterInvalidException(@NonNull String parameter, @NonNull String message) {
        this(parameter, message, null);
    }

    public ParameterInvalidException(@NonNull String parameter, @NonNull String message, Throwable cause) {
        this(ImmutableMap.of(parameter, message), cause);
    }

    public ParameterInvalidException(@NonNull Map<String, String> errors) {
        this(errors, null);
    }

    public ParameterInvalidException(@NonNull Map<String, String> errors, Throwable cause) {
        super(CODE, errors, String.valueOf(errors), cause);
    }

    /**
     * 将参数无效异常转换成结果对象
     *
     * @return 结果对象实例
     */
    public Result<?> result() {
        return Result.of(CODE, this.getData(), "Invalid parameter");
    }

    /**
     * 构建参数无效异常结果
     *
     * @param errors 参数异常映射表
     * @return 结果对象实例
     */
    public static Result<?> result(@NonNull Map<String, String> errors) {
        return Result.of(CODE, errors, "Invalid parameter");
    }
}
