package com.chelvc.framework.common.model;

import com.chelvc.framework.common.util.ObjectUtils;

/**
 * 属性复制接口
 *
 * @author Woody
 * @date 2024/1/30
 */
public interface Copier {
    /**
     * 将当前对象属性值复制到目标对象实例
     *
     * @param target 目标对象实例
     * @param <T>    目标对象类型
     */
    default <T> void to(T target) {
        ObjectUtils.copying(this, target);
    }

    /**
     * 将当前对象属性值复制到目标对象
     *
     * @param target 目标对象
     * @param <T>    目标对象类型
     * @return 目标对象实例
     */
    default <T> T to(Class<T> target) {
        return ObjectUtils.copying(this, target);
    }

    /**
     * 将对象属性值复制到当前对象
     *
     * @param object 被复制对象实例
     * @param <T>    被复制对象类型
     */
    default <T> void of(T object) {
        ObjectUtils.copying(object, this);
    }
}
