package com.chelvc.framework.common.util;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;

import lombok.NonNull;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.codec.digest.MessageDigestAlgorithms;

/**
 * 编解码工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class CodecUtils {
    private CodecUtils() {
    }

    /**
     * MD5加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String md5(String plaintext) {
        return encode(plaintext, MessageDigestAlgorithms.MD5);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String sha256(String plaintext) {
        return encode(plaintext, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static String encode(String plaintext, @NonNull String algorithm) {
        if (plaintext == null) {
            return null;
        }
        MessageDigest digest = DigestUtils.getDigest(algorithm);
        byte[] bytes = digest.digest(plaintext.getBytes(StandardCharsets.UTF_8));
        return Hex.encodeHexString(bytes);
    }
}
