package com.chelvc.framework.database.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * 创建或者更新注解
 *
 * @author Woody
 * @date 2024/3/5
 */
@Inherited
@Documented
@Target(ElementType.FIELD)
@Retention(RetentionPolicy.RUNTIME)
public @interface CreateUpdate {
    /**
     * 判断字段是否启用更新
     *
     * @return true/false
     */
    boolean enabled() default true;

    /**
     * 获取字段更新方式
     *
     * @return 更新方式
     */
    Mode mode() default Mode.DEFAULT;

    /**
     * 更新方式枚举
     */
    enum Mode {
        /**
         * 原值 + 新值
         */
        ADD,

        /**
         * 原值 - 新值
         */
        SUBTRACT,

        /**
         * 原值 * 新值
         */
        MULTIPLY,

        /**
         * 原值 / 新值
         */
        DIVIDE,

        /**
         * 默认等于新值
         */
        DEFAULT;
    }
}
