package com.chelvc.framework.database.interceptor;

import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.common.exception.ParameterInvalidException;
import com.chelvc.framework.common.model.Result;
import com.chelvc.framework.common.util.ErrorUtils;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 数据库异常处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@RestControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class DatabaseExceptionInterceptor {
    /**
     * 数据持久化异常转换
     *
     * @param request Http请求对象
     * @param e       异常对象
     * @return 请求结果
     */
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    @ExceptionHandler(MyBatisSystemException.class)
    public Result<?> exception(HttpServletRequest request, MyBatisSystemException e) {
        ParameterInvalidException parameterInvalidException = ErrorUtils.lookup(e, ParameterInvalidException.class);
        if (parameterInvalidException != null) {
            LoggingContextHolder.warn(log, request, e);
            return parameterInvalidException.result();
        }
        LoggingContextHolder.error(log, request, e);
        return Result.failure();
    }
}
