package com.chelvc.framework.database.interceptor;

import java.math.BigInteger;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.entity.Deletable;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;

/**
 * 借助租户隔离功能实现删除数据排除拦截，数据查询自动过滤已删除数据
 *
 * @author Woody
 * @date 2024/1/30
 */
public class DeletedExcludeHandler implements TenantLineHandler {
    /**
     * 删除标记字段名称
     */
    public static final String DELETED_COLUMN_NAME = "deleted";

    /**
     * 未删除值对象
     */
    private static final LongValue VALUE = new LongValue(0) {
        @Override
        public long getValue() {
            return 0;
        }

        @Override
        public BigInteger getBigIntegerValue() {
            return BigInteger.ZERO;
        }
    };

    @Override
    public Expression getTenantId() {
        return VALUE;
    }

    @Override
    public String getTenantIdColumn() {
        return DELETED_COLUMN_NAME;
    }

    @Override
    public boolean ignoreTable(String tableName) {
        TableInfo table = DatabaseContextHolder.getTable(tableName);
        return table == null || !Deletable.class.isAssignableFrom(table.getEntityType());
    }
}
