package com.chelvc.framework.database.interceptor;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.model.Modification;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;

/**
 * 对象更新信息字段类型处理实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class ModificationTypeHandler extends BaseTypeHandler<Modification> {
    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, Modification parameter, JdbcType jdbcType)
            throws SQLException {
        if (parameter == null) {
            ps.setNull(i, Types.VARCHAR);
        } else {
            ps.setString(i, JacksonContextHolder.serialize(parameter));
        }
    }

    @Override
    public Modification getNullableResult(ResultSet rs, String columnName) throws SQLException {
        return JacksonContextHolder.deserialize(rs.getString(columnName), Modification.class);
    }

    @Override
    public Modification getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        return JacksonContextHolder.deserialize(rs.getString(columnIndex), Modification.class);
    }

    @Override
    public Modification getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        return JacksonContextHolder.deserialize(cs.getString(columnIndex), Modification.class);
    }
}
