package com.chelvc.framework.database.support;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.annotation.CreateUpdate;
import lombok.NonNull;

/**
 * 插入或修改方法实现
 *
 * @author Woody
 * @date 2024/2/27
 */
public class InsertUpdateMethod extends AbstractInsertMethod {
    public InsertUpdateMethod() {
        super("insertUpdate");
    }

    @Override
    protected String initializeMethodScript(@NonNull TableInfo table) {
        List<TableFieldInfo> fields = table.getFieldList();
        String columns = this.field2column(table, fields);
        String properties = this.field2property(table, fields);
        String updates = fields.stream().map(field -> {
            CreateUpdate annotation = field.getField().getAnnotation(CreateUpdate.class);
            if (annotation != null && !annotation.enabled()) {
                return null;
            }
            String column = field.getColumn();
            StringBuilder buffer = new StringBuilder().append(column).append(" = ");
            CreateUpdate.Mode mode = ObjectUtils.ifNull(annotation, CreateUpdate::mode);
            if (mode == CreateUpdate.Mode.ADD) {
                buffer.append(column).append(" + ");
            } else if (mode == CreateUpdate.Mode.SUBTRACT) {
                buffer.append(column).append(" - ");
            } else if (mode == CreateUpdate.Mode.MULTIPLY) {
                buffer.append(column).append(" * ");
            } else if (mode == CreateUpdate.Mode.DIVIDE) {
                buffer.append(column).append(" / ");
            }
            return buffer.append("VALUES(").append(column).append(")");
        }).filter(Objects::nonNull).collect(Collectors.joining(StringPool.COMMA));
        return String.format(
                "<script>\nINSERT IGNORE INTO %s (%s) VALUES (%s)\n ON DUPLICATE KEY UPDATE %s\n</script>",
                table.getTableName(), columns, properties, updates
        );
    }
}
