package com.chelvc.framework.dubbo.interceptor;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterWrapper;
import org.apache.dubbo.common.constants.CommonConstants;
import org.apache.dubbo.common.extension.Activate;
import org.apache.dubbo.rpc.AsyncRpcResult;
import org.apache.dubbo.rpc.Filter;
import org.apache.dubbo.rpc.Invocation;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.Result;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.RpcInvocation;

/**
 * Dubbo 消费者处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Activate(group = CommonConstants.CONSUMER)
public class DubboConsumerInterceptor implements Filter {
    /**
     * 查找接口对象适配器
     *
     * @param invocation Dubbo接口调用信息
     * @return Dubbo接口对象适配器
     */
    private DubboObjectAdapterWrapper<?, Object> lookupObjectAdapter(Invocation invocation) {
        if (!(invocation instanceof RpcInvocation)) {
            return null;
        }

        // 根据接口返回类型查找对象适配器
        Class<?> returnType = ((RpcInvocation) invocation).getReturnType();
        if (returnType == void.class) {
            return null;
        }
        DubboObjectAdapterWrapper<?, Object> wrapper = DubboObjectAdapterHolder.lookupObjectAdapter(returnType);
        if (wrapper != null) {
            // 设置适配目标对象类型
            ((RpcInvocation) invocation).setReturnType(wrapper.getTarget());
            ((RpcInvocation) invocation).getReturnTypes()[0] = wrapper.getTarget();
        }
        return wrapper;
    }

    @Override
    public Result invoke(Invoker<?> invoker, Invocation invocation) throws RpcException {
        // 初始化会话信息
        RpcContext.getClientAttachment().setObjectAttachment(
                SessionContextHolder.NAME, SessionContextHolder.getSession(false)
        );

        // 查找对象适配器
        DubboObjectAdapterWrapper<?, Object> wrapper = this.lookupObjectAdapter(invocation);

        try {
            // 接口调用
            Result result = invoker.invoke(invocation);
            if (wrapper == null) {
                return result;
            }

            // 对象解包
            Object value = wrapper.getAdapter().unwrap(result.getValue());
            return AsyncRpcResult.newDefaultAsyncResult(value, invocation);
        } finally {
            // 清理请求上下文
            RpcContext.getClientAttachment().clearAttachments();
        }
    }
}
