package com.chelvc.framework.feign.interceptor;

import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.model.Result;
import feign.Response;
import feign.Util;
import feign.codec.ErrorDecoder;

/**
 * Feign调用异常处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class FeignExceptionInterceptor implements ErrorDecoder {
    @Override
    public Exception decode(String method, Response response) {
        byte[] bytes;
        try (InputStream input = response.body().asInputStream()) {
            bytes = Util.toByteArray(input);
        } catch (IOException e) {
            return e;
        }
        Result<?> result = Objects.requireNonNull(JacksonContextHolder.deserialize(bytes, Result.class));
        return new FrameworkException(result.getCode(), result.getData(), result.getMessage());
    }
}
