package com.chelvc.framework.feign.interceptor;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * Feign请求头处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
public class FeignHeaderInterceptor implements RequestInterceptor {
    @Override
    public void apply(RequestTemplate template) {
        // 拷贝当前会话请求头
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            template.header("x-real-ip", session.getHost());
            template.header(SessionContextHolder.HEADER_ID,
                    (String) ObjectUtils.ifNull(session.getId(), String::valueOf));
            template.header(SessionContextHolder.HEADER_SCOPE, session.getScope());
            template.header(SessionContextHolder.HEADER_DEVICE, session.getDevice());
            template.header(SessionContextHolder.HEADER_CHANNEL, session.getChannel());
            template.header(SessionContextHolder.HEADER_PLATFORM,
                    ObjectUtils.ifNull(session.getPlatform(), Platform::name));
            template.header(SessionContextHolder.HEADER_TERMINAL,
                    ObjectUtils.ifNull(session.getTerminal(), Terminal::name));
            template.header(SessionContextHolder.HEADER_VERSION, session.getVersion());
            template.header(SessionContextHolder.HEADER_ANONYMOUS, String.valueOf(session.isAnonymous()));
            template.header(SessionContextHolder.HEADER_TIMESTAMP,
                    (String) ObjectUtils.ifNull(session.getTimestamp(), String::valueOf));
            template.header(SessionContextHolder.HEADER_FINGERPRINT, session.getFingerprint());
        }
        if (log.isDebugEnabled()) {
            log.debug("Feign request headers: {}", template.headers());
        }
    }
}
