package com.chelvc.framework.jpush.config;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 极光配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Data
@Configuration
@ConfigurationProperties("jpush")
public class JPushProperties {
    /**
     * 是否是生产环境
     */
    private boolean production;

    /**
     * 消息推送配置
     */
    private final Push push = new Push();

    /**
     * 登录认证配置
     */
    private final Login login = new Login();

    /**
     * 消息推送配置
     */
    @Data
    public static class Push {
        /**
         * 应用标识
         */
        private String appkey;

        /**
         * 编解码密钥
         */
        private String secret;

        /**
         * 华为配置
         */
        private final Attribute huawei = new Attribute();

        /**
         * 荣耀配置
         */
        private final Attribute honor = new Attribute();

        /**
         * 小米配置
         */
        private final Attribute xiaomi = new Attribute();

        /**
         * OPPO配置
         */
        private final Attribute oppo = new Attribute();

        /**
         * VIVO配置
         */
        private final Attribute vivo = new Attribute();
    }

    /**
     * 登录认证配置
     */
    @Data
    public static class Login {
        /**
         * 编解码密钥
         */
        private String secret;

        /**
         * 认证信息
         */
        private String authorization;
    }

    /**
     * 属性配置
     */
    @Data
    public static class Attribute {
        /**
         * 渠道
         */
        private String channel;

        /**
         * 重要性
         */
        private String importance;

        /**
         * 分布
         */
        private String distribution;

        /**
         * 分类
         */
        private Integer classification;
    }
}
