package com.chelvc.framework.oauth.token;

import java.util.Objects;

import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.oauth.context.OAuthContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2Error;
import org.springframework.security.oauth2.core.OAuth2TokenValidatorResult;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Component;

/**
 * 基于Redis的令牌有效性验证实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@ConditionalOnClass(RedisContextHolder.class)
public class RedisTokenActiveValidator implements TokenActiveValidator {
    @Override
    public OAuth2TokenValidatorResult validate(Jwt jwt) {
        String key = OAuthContextHolder.key(OAuthContextHolder.getId(jwt));
        String token = (String) RedisContextHolder.getDefaultTemplate().opsForValue().get(key);
        if (StringUtils.isEmpty(token)) {
            throw new OAuth2AuthenticationException(new OAuth2Error("TOKEN_EXPIRED", "Token has expired", null));
        } else if (!Objects.equals(token, jwt.getTokenValue())) {
            // 判断令牌是否相同，如果不相同则标识令牌已经被重置（账号已在其他地方登录）
            throw new OAuth2AuthenticationException(new OAuth2Error("TOKEN_REPLACED", "Token has replaced", null));
        }
        return OAuth2TokenValidatorResult.success();
    }
}
