package com.chelvc.framework.oauth.token;

import java.time.Duration;

import com.chelvc.framework.oauth.context.OAuthContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

/**
 * 基于Redis令牌管理器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class RedisTokenStore extends JwtTokenStore {
    public RedisTokenStore(@NonNull JwtAccessTokenConverter jwtAccessTokenConverter) {
        super(jwtAccessTokenConverter);
    }

    @Override
    public void storeAccessToken(OAuth2AccessToken token, OAuth2Authentication authentication) {
        Object principal = authentication.getPrincipal();
        if (principal instanceof UserDetails) {
            principal = ((UserDetails) principal).getUsername();
        }
        String key = OAuthContextHolder.key(principal);
        Duration duration = RedisContextHolder.duration(token.getExpiration());
        RedisContextHolder.getDefaultTemplate().opsForValue().set(key, token.getValue(), duration);
    }
}
