package com.chelvc.framework.redis.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Redis Stream 消费者注解
 *
 * @author Woody
 * @date 2024/1/30
 */
@Inherited
@Documented
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
public @interface RedisMQConsumer {
    /**
     * 获取消息主题
     *
     * @return 消息主题
     */
    String topic();

    /**
     * 获取消费者分组
     *
     * @return 消费者分组
     */
    String group();

    /**
     * 批量获取消息数量
     *
     * @return 消息数量
     */
    int batch() default 100;

    /**
     * 获取并发线程数量
     *
     * @return 并发线程数量
     */
    int concurrency() default 20;
}
