package com.chelvc.framework.rocketmq.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * RocketMQ消费者注解
 *
 * @author Woody
 * @date 2024/1/30
 */
@Inherited
@Documented
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
public @interface RocketMQConsumer {
    /**
     * 获取消息主题
     *
     * @return 消息主题
     */
    String topic();

    /**
     * 获取消费者分组
     *
     * @return 消费者分组
     */
    String group();

    /**
     * 获取消息过滤标签
     *
     * @return 消息过滤标签
     */
    String tag() default "*";

    /**
     * 批量获取消息数量
     *
     * @return 消息数量
     */
    int batch() default 1;

    /**
     * 消息处理时间周期（秒），仅batch > 1时生效
     *
     * @return 时间周期
     */
    int duration() default 15;

    /**
     * 消息拉取时间间隔（秒），仅batch > 1时生效
     *
     * @return 时间间隔
     */
    int interval() default 30;

    /**
     * 获取并发线程数量，仅batch = 1时生效
     *
     * @return 并发线程数量
     */
    int concurrency() default 20;
}
