package com.chelvc.framework.rocketmq.context;

import java.util.function.Function;

import lombok.NonNull;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * 事务消息发送包装器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class TransactionalMessageSending extends MessageSending {
    /**
     * 本地事务执行方法
     */
    protected final Function<String, Boolean> executor;

    public TransactionalMessageSending(@NonNull Producer producer, @NonNull Message message,
                                       @NonNull Function<String, Boolean> executor) {
        super(producer, message);
        this.executor = executor;
    }

    @Override
    public void retry() throws ClientException {
        Transaction transaction = this.producer.beginTransaction();
        MessageId id = this.producer.send(this.message, transaction).getMessageId();
        if (Boolean.TRUE.equals(this.executor.apply(String.valueOf(id)))) {
            transaction.commit();
        } else {
            transaction.rollback();
        }
    }
}
