package com.chelvc.framework.security.interceptor;

import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 安全异常处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@RestControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class SecurityExceptionInterceptor {
    /**
     * 拒绝访问异常转换
     *
     * @param request Http请求对象
     * @param e       异常对象
     * @return 请求结果
     */
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ExceptionHandler(AccessDeniedException.class)
    public Result<?> exception(HttpServletRequest request, AccessDeniedException e) {
        LoggingContextHolder.warn(log, request, e);
        return SessionContextHolder.result(HttpStatus.FORBIDDEN, e.getMessage());
    }
}
