package com.chelvc.framework.security.serializer;

import java.io.IOException;

import com.chelvc.framework.common.util.DesensitizeUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Desensitize;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import lombok.extern.slf4j.Slf4j;

/**
 * JSON数据脱敏序列化处理器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class JacksonDesensitizeSerializer extends StdSerializer<String> implements ContextualSerializer {
    private DesensitizeUtils.Mode mode;

    public JacksonDesensitizeSerializer() {
        super(String.class);
    }

    @Override
    public void serialize(String plaintext, JsonGenerator generator, SerializerProvider provider)
            throws IOException {
        if (this.mode == null || StringUtils.isEmpty(plaintext)) {
            generator.writeString(plaintext);
        } else {
            String ciphertext = this.mode.desensitize(plaintext);
            if (log.isDebugEnabled()) {
                log.debug("Data desensitize: {} -> {}", plaintext, ciphertext);
            }
            generator.writeString(ciphertext);
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider provider, BeanProperty property) {
        Desensitize annotation = property.getAnnotation(Desensitize.class);
        String expression = ObjectUtils.ifNull(annotation, Desensitize::expression);
        this.mode = StringUtils.ifEmpty(expression, DesensitizeUtils::analyse);
        return this;
    }
}
