package com.chelvc.framework.sms.config;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.sms.CaptchaSmsHandler;
import com.chelvc.framework.sms.NormalSmsHandler;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.chelvc.framework.sms.support.AliyunSmsHandler;
import com.chelvc.framework.sms.support.DefaultCaptchaSmsHandler;
import com.chelvc.framework.sms.support.DelegatingNormalSmsHandler;
import com.chelvc.framework.sms.support.DelegatingTemplateSmsHandler;
import com.chelvc.framework.sms.support.TencentSmsHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 短信配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SmsConfigurer {
    private final SmsProperties properties;

    /**
     * 初始化短信处理器实例
     *
     * @param client 客户端配置
     * @return 短信处理器实例
     */
    private Object initializeSmsHandler(SmsProperties.Client client) {
        SmsProperties.Channel channel = client.getChannel();
        if (channel == SmsProperties.Channel.ALIYUN) {
            return new AliyunSmsHandler(client);
        } else if (channel == SmsProperties.Channel.TENCENT) {
            return new TencentSmsHandler(client);
        }
        throw new UnsupportedOperationException(String.valueOf(channel));
    }

    @Bean
    public NormalSmsHandler normalSmsHandler() {
        List<NormalSmsHandler> handlers = this.properties.getClients().stream().map(client -> {
            Object handler = this.initializeSmsHandler(client);
            return handler instanceof NormalSmsHandler ? (NormalSmsHandler) handler : null;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        return ObjectUtils.isEmpty(handlers) ? null : new DelegatingNormalSmsHandler(handlers);
    }

    @Bean
    public TemplateSmsHandler templateSmsHandler() {
        List<TemplateSmsHandler> handlers = this.properties.getClients().stream().map(client -> {
            Object handler = this.initializeSmsHandler(client);
            return handler instanceof TemplateSmsHandler ? (TemplateSmsHandler) handler : null;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        return ObjectUtils.isEmpty(handlers) ? null : new DelegatingTemplateSmsHandler(handlers);
    }

    @Bean
    public CaptchaSmsHandler captchaSmsHandler() {
        return new DefaultCaptchaSmsHandler(this.templateSmsHandler(), this.properties.getCaptcha());
    }
}
