package com.chelvc.framework.sms.config;

import java.util.Collections;
import java.util.List;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 阿里云短信配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Data
@Configuration
@ConfigurationProperties("sms")
public class SmsProperties {
    /**
     * 验证码配置
     */
    private final Captcha captcha = new Captcha();

    /**
     * 短信客户端配置
     */
    private List<Client> clients = Collections.emptyList();

    /**
     * 短信渠道枚举
     */
    public enum Channel {
        /**
         * 阿里云
         */
        ALIYUN,

        /**
         * 腾讯云
         */
        TENCENT;
    }

    /**
     * 客户端配置
     */
    @Data
    public static class Client {
        /**
         * ID
         */
        private String id;

        /**
         * 密钥
         */
        private String secret;

        /**
         * 地区标识
         */
        private String region;

        /**
         * 短信签名
         */
        private String signature;

        /**
         * 短信渠道
         */
        private Channel channel;
    }

    /**
     * 验证码配置
     */
    @Data
    public static class Captcha {
        /**
         * 验证码模版
         */
        private String template;

        /**
         * 验证码长度
         */
        private int length = 6;

        /**
         * 验证码发送间隔时间（秒）
         */
        private int interval = 60;

        /**
         * 验证码有效时间（秒）
         */
        private int expiration = 10 * 60;
    }
}
