package com.chelvc.framework.sms.support;

import java.util.Map;
import java.util.Objects;

import com.aliyun.dysmsapi20170525.Client;
import com.aliyun.dysmsapi20170525.models.SendSmsRequest;
import com.aliyun.dysmsapi20170525.models.SendSmsResponse;
import com.aliyun.dysmsapi20170525.models.SendSmsResponseBody;
import com.aliyun.teaopenapi.models.Config;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.sms.Captcha;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.chelvc.framework.sms.config.SmsProperties;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 基于阿里云短信操作实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class AliyunSmsHandler implements TemplateSmsHandler {
    private final Client client;
    private final String signature;

    public AliyunSmsHandler(@NonNull SmsProperties.Client properties) {
        Config config = new Config().setAccessKeyId(properties.getId()).setAccessKeySecret(properties.getSecret())
                .setEndpoint("dysmsapi.aliyuncs.com");
        try {
            this.client = new Client(config);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        this.signature = properties.getSignature();
    }

    @Override
    public void send(@NonNull String template, @NonNull Captcha captcha) {
        this.send(captcha.getMobile(), template, ImmutableMap.of("code", captcha.getCode()));
    }

    @Override
    public void send(String mobile, String template, String... parameters) {
        throw new UnsupportedOperationException("Not support method");
    }

    @Override
    public void send(@NonNull String mobile, @NonNull String template, @NonNull Map<String, ?> parameters) {
        String param = JacksonContextHolder.serialize(parameters);
        SendSmsRequest request = new SendSmsRequest().setPhoneNumbers(mobile).setSignName(this.signature)
                .setTemplateCode(template).setTemplateParam(param);
        SendSmsResponse response;
        try {
            response = this.client.sendSms(request);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        SendSmsResponseBody body = response.getBody();
        if (!Objects.equals(body.getCode(), "OK")) {
            if (Objects.equals(body.getCode(), "isv.DAY_LIMIT_CONTROL")
                    || Objects.equals(body.getCode(), "isv.MONTH_LIMIT_CONTROL")
                    || Objects.equals(body.getCode(), "isv.BUSINESS_LIMIT_CONTROL")) {
                throw new ResourceUnavailableException("短信发送次数已达上限");
            }
            throw new ResourceUnavailableException(body.getMessage());
        }
    }
}
