package com.chelvc.framework.sms.support;

import java.util.Map;

import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.sms.Captcha;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.chelvc.framework.sms.config.SmsProperties;
import com.github.qcloudsms.SmsSingleSender;
import com.github.qcloudsms.SmsSingleSenderResult;
import com.github.qcloudsms.httpclient.PoolingHTTPClient;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 基于腾讯云的模版短信操作实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class TencentSmsHandler implements TemplateSmsHandler {
    private final SmsSingleSender sender;
    private final String region;
    private final String signature;

    public TencentSmsHandler(@NonNull SmsProperties.Client properties) {
        int appid = Integer.parseInt(properties.getId());
        this.sender = new SmsSingleSender(appid, properties.getSecret(), new PoolingHTTPClient());
        this.region = properties.getRegion();
        this.signature = properties.getSignature();
    }

    @Override
    public void send(@NonNull String template, @NonNull Captcha captcha) {
        this.send(captcha.getMobile(), template, captcha.getCode());
    }

    @Override
    public void send(@NonNull String mobile, @NonNull String template, @NonNull String... parameters) {
        SmsSingleSenderResult result;
        try {
            result = this.sender.sendWithParam(this.region, mobile, Integer.parseInt(template),
                    parameters, this.signature, null, null);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (result.result != 0) {
            if (result.result == 1023 || result.result == 1025) {
                throw new ResourceUnavailableException("短信发送次数已达上限");
            }
            throw new ResourceUnavailableException(result.errMsg);
        }
    }

    @Override
    public void send(String mobile, String template, Map<String, ?> parameters) {
        throw new UnsupportedOperationException("Not support method");
    }
}
