package com.chelvc.framework.upload.config;

import java.util.List;
import java.util.stream.Collectors;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.upload.UploadHandler;
import com.chelvc.framework.upload.support.DelegatingUploadHandler;
import com.chelvc.framework.upload.support.SimpleUploadHandler;
import com.chelvc.framework.upload.support.TencentUploadHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 文件上传配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UploadConfigurer {
    private final UploadProperties properties;

    /**
     * 初始化文件上传处理器实例
     *
     * @param client 客户端配置
     * @return 文件上传处理器实例
     */
    private UploadHandler initializeUploadHandler(UploadProperties.Client client) {
        UploadProperties.Channel channel = client.getChannel();
        if (channel == UploadProperties.Channel.LOCAL) {
            return new SimpleUploadHandler(client);
        } else if (channel == UploadProperties.Channel.TENCENT) {
            return new TencentUploadHandler(client, this.properties.getHttpclient());
        }
        throw new UnsupportedOperationException(String.valueOf(channel));
    }

    @Bean(destroyMethod = "destroy")
    public UploadHandler uploadHandler() {
        List<UploadHandler> handlers = this.properties.getClients().stream()
                .map(this::initializeUploadHandler).collect(Collectors.toList());
        return ObjectUtils.isEmpty(handlers) ? null : new DelegatingUploadHandler(handlers);
    }
}
