package com.chelvc.framework.upload.config;

import java.util.Collections;
import java.util.List;

import com.chelvc.framework.base.config.HttpClientProperties;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 文件上传配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Data
@Configuration
@ConfigurationProperties("upload")
public class UploadProperties {
    /**
     * 文件上传客户端配置
     */
    private List<Client> clients = Collections.emptyList();

    /**
     * http链接配置
     */
    private final HttpClientProperties httpclient = new HttpClientProperties();

    /**
     * 文件上传渠道枚举
     */
    public enum Channel {
        /**
         * 本地
         */
        LOCAL,

        /**
         * 腾讯云
         */
        TENCENT;
    }

    /**
     * 客户端配置
     */
    @Data
    public static class Client {
        /**
         * 应用标识
         */
        private String id;

        /**
         * 应用密钥
         */
        private String secret;

        /**
         * 地区标识
         */
        private String region;

        /**
         * 存储路径
         */
        private String path;

        /**
         * 文件访问域名
         */
        private String domain;

        /**
         * 文件上传渠道
         */
        private Channel channel;
    }
}
