package com.chelvc.framework.upload.support;

import java.io.File;
import java.io.InputStream;
import java.util.Objects;

import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.upload.UploadHandler;
import com.chelvc.framework.upload.config.UploadProperties;
import lombok.NonNull;

/**
 * 文件上传简单实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class SimpleUploadHandler implements UploadHandler {
    private final String path;
    private final String domain;

    public SimpleUploadHandler(@NonNull UploadProperties.Client properties) {
        this.path = Objects.requireNonNull(properties.getPath());
        this.domain = Objects.requireNonNull(properties.getDomain());
    }

    @Override
    public String upload(@NonNull File file) {
        String suffix = FileUtils.getSuffix(file);
        String filename = StringUtils.uuid() + StringUtils.ifEmpty(suffix, name -> "." + name, StringUtils.EMPTY);
        FileUtils.write(new File(this.path, filename), file);
        return HttpUtils.url(this.domain, filename);
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix) {
        return this.upload(stream, suffix, FileUtils.getLength(stream));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix, long length) {
        String filename = StringUtils.uuid() + StringUtils.ifEmpty(suffix, name -> "." + name, StringUtils.EMPTY);
        FileUtils.write(new File(this.path, filename), stream);
        return HttpUtils.url(this.domain, filename);
    }
}
