package com.chelvc.framework.wechat;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

/**
 * 微信公众号操作接口
 *
 * @author Woody
 * @date 2024/1/30
 */
public interface WechatPublicHandler {
    /**
     * 获取配置ID
     *
     * @return 配置ID
     */
    String getId();

    /**
     * 获取AppID
     *
     * @return AppID
     */
    String getAppid();

    /**
     * 获取微信公众号访问令牌
     *
     * @return 访问令牌
     */
    String getAccessToken();

    /**
     * 刷新微信访问令牌
     *
     * @return 访问令牌
     */
    String refreshAccessToken();

    /**
     * 根据临时令牌获取微信令牌
     *
     * @param code 临时令牌
     * @return 微信令牌
     */
    WechatWebToken code2token(String code);

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @return 明文
     */
    String decrypt(String ciphertext);

    /**
     * 获取微信公众号用户信息
     *
     * @param openid openid
     * @return 微信用户信息
     */
    WechatUser getUser(String openid);

    /**
     * 遍历所有关注用户openid
     *
     * @param consumer openid回调接口
     */
    void iterateOpenids(Consumer<List<String>> consumer);

    /**
     * 遍历所有关注用户openid
     *
     * @param consumer openid回调接口
     */
    void iterateOpenids(BiConsumer<List<String>, Integer> consumer);

    /**
     * 批量获取关注者信息
     *
     * @param openids 关注者openid集合
     * @return 关注者信息列表
     */
    List<WechatPublicSubscriber> getSubscribers(Collection<String> openids);

    /**
     * 数据签名
     *
     * @param timestamp 时间戳
     * @param elements  签名要素
     * @return 签名信息
     */
    String sign(long timestamp, String... elements);

    /**
     * 推送消息
     *
     * @param openid     用户openid
     * @param template   消息模板标识
     * @param parameters 模版参数
     * @return 消息ID
     */
    long push(String openid, String template, Map<?, ?> parameters);

    /**
     * 发送消息
     *
     * @param openid     用户openid
     * @param template   消息模板标识
     * @param redirect   重定向路径
     * @param parameters 模版参数
     * @return 消息ID
     */
    long push(String openid, String template, String redirect, Map<?, ?> parameters);
}
